@@ -6,6 +6,7 @@ use warnings;
 use Cwd;
 use File::Copy;
 use File::Path qw(mkpath remove_tree);
+use Getopt::Long qw(:config posix_default bundling no_ignore_case);
 use IPC::Open3;
 use Symbol 'gensym';
 use UNIVERSAL;
@@ -13,7 +14,7 @@ use UNIVERSAL;
 use Data::Dumper qw(Dumper);
 
 #  NiHTest -- package to run regression tests
-#  Copyright (C) 2002-2015 Dieter Baron and Thomas Klausner
+#  Copyright (C) 2002-2016 Dieter Baron and Thomas Klausner
 #
 #  This file is part of ckmame, a program to check rom sets for MAME.
 #  The authors can be contacted at <ckmame@nih.at>
@@ -312,7 +313,7 @@ sub runtest {
 
 	my @failed = ();
 	
-	if ($self->{exit_status} != $self->{test}->{return} // 0) {
+	if ($self->{exit_status} != ($self->{test}->{return} // 0)) {
 		push @failed, 'exit status';
 		if ($self->{verbose}) {
 			print "Unexpected exit status:\n";
@@ -349,12 +350,25 @@ sub runtest {
 
 sub setup {
 	my ($self, @argv) = @_;
-	
-	if (scalar(@argv) != 1) {
-		print STDERR "Usage: $0 testcase\n";
+
+	my @save_argv = @ARGV;
+	@ARGV = @argv;
+	my $ok = GetOptions(
+		'help|h' => \my $help,
+		'keep-broken' => \$self->{keep_broken},
+		'no-cleanup' => \$self->{no_cleanup},
+		# 'run-gdb' => \$self->{run_gdb},
+		'setup-only' => \$self->{setup_only},
+		'verbose|v' => \$self->{verbose}
+	);
+	@argv = @ARGV;
+	@ARGV = @save_argv;
+
+	if (!$ok || scalar(@argv) != 1 || $help) {
+		print STDERR "Usage: $0 [-hv] [--keep-broken] [--no-cleanup] [--setup-only] testcase\n";
 		exit(1);
 	}
-	
+
 	my $testcase = shift @argv;
 
 	$testcase .= '.test' unless ($testcase =~ m/\.test$/);
@@ -369,6 +383,8 @@ sub setup {
 	$self->die("error in test case definition") unless $self->parse_case($testcase_file);
 	
 	$self->check_features_requirement() if ($self->{test}->{features});
+
+	$self->end_test('SKIP') if ($self->{test}->{preload} && $^O eq 'darwin');
 }
 
 
@@ -483,6 +499,7 @@ sub compare_file($$$) {
 	return $ok;
 }
 
+
 sub compare_files() {
 	my ($self) = @_;
 	
@@ -662,9 +679,24 @@ sub parse_args {
 		}
 		my @types = split /\s+/, $type;
 		my @strs = split /\s+/, $str;
-		
-		if (!$ellipsis && scalar(@types) != scalar(@strs)) {
-			$self->warn_file_line("expected " . (scalar(@types)) . " arguments, got " . (scalar(@strs)));
+		my $optional = 0;
+		for (my $i = scalar(@types) - 1; $i >= 0; $i--) {
+			last unless ($types[$i] =~ m/(.*)\?$/);
+			$types[$i] = $1;
+			$optional++;
+		}
+
+		if ($ellipsis && $optional > 0) {
+			# TODO: check this when registering a directive
+			$self->warn_file_line("can't use ellipsis together with optional arguments");
+			return undef;
+		}
+		if (!$ellipsis && (scalar(@strs) < scalar(@types) - $optional || scalar(@strs) > scalar(@types))) {
+			my $expected = scalar(@types);
+			if ($optional > 0) {
+				$expected = ($expected - $optional) . "-$expected";
+			}
+			$self->warn_file_line("expected $expected arguments, got " . (scalar(@strs)));
 			return undef;
 		}
 		
@@ -737,8 +769,8 @@ sub parse_case() {
 		}
 		
 		my $args = $self->parse_args($def->{type}, $argstring);
-
-		if (!defined($args)) {
+            
+		unless (defined($args)) {
 			$ok = 0;
 			next;
 		}
@@ -870,10 +902,10 @@ sub run_hook {
 	
 	return $ok;
 }
+sub args_decode {
 
 
-sub backslash_decode {
-	my ($str) = @_;
+	my ($str, $srcdir) = @_;
 
 	if ($str =~ m/\\/) {
 		$str =~ s/\\a/\a/gi;
@@ -888,19 +920,25 @@ sub backslash_decode {
 		$str =~ s/\\(.)/$1/g;
 	}
 
+	if ($srcdir !~ m,^/,) {
+		$srcdir = "../$srcdir";
+	}
+
+	if ($str =~ m/^\$srcdir(.*)/) {
+		$str = "$srcdir$1";
+	}
+
 	return $str;
 }
 
 
 sub run_program {
 	my ($self) = @_;
-
 	goto &pipein_win32 if $^O eq 'MSWin32' && $self->{test}->{pipein};
-
 	my ($stdin, $stdout, $stderr);
 	$stderr = gensym;
 
-	my @cmd = ('../' . $self->{test}->{program}, map ({ backslash_decode($_); } @{$self->{test}->{args}}));
+	my @cmd = ('../' . $self->{test}->{program}, map ({ args_decode($_, $self->{srcdir}); } @{$self->{test}->{args}}));
 
 	### TODO: catch errors?
 	
@@ -956,7 +994,7 @@ sub run_program {
 sub pipein_win32() {
 	my ($self) = @_;
 
-	my $cmd = "$self->{test}->{pipein}| ..\\$self->{test}->{program} " . join(' ', map ({ backslash_decode($_); } @{$self->{test}->{args}}));
+	my $cmd = "$self->{test}->{pipein}| ..\\$self->{test}->{program} " . join(' ', map ({ args_decode($_, $self->{srcdir}); } @{$self->{test}->{args}}));
 	my ($success, $error_message, $full_buf, $stdout_buf, $stderr_buf) = IPC::Cmd::run(command => $cmd);
 	if (!$success) {
 		### TODO: catch errors?
